/*
	rpipic - A PIC Programmer for the Raspberry Pi
	Copyright (C) 2012  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/


#ifndef BOARD_HPP
#define BOARD_HPP

#include "gpio.hpp"

class Board
{
	public:
		virtual void SetVddHigh(bool high) = 0;
		virtual void SetVppHigh(bool high) = 0;
		virtual void SetClockHigh(bool high) = 0;
		virtual void SetDataHigh(bool high) = 0;
		virtual bool GetDataHigh() = 0;
};

class NullBoard : public Board
{
	public:
		virtual void SetVddHigh(bool high){};
		virtual void SetVppHigh(bool high){};
		virtual void SetClockHigh(bool high){};
		virtual void SetDataHigh(bool high){};
		virtual bool GetDataHigh(){return false;};
};

class Breadboard : public Board
{
	public:
		Breadboard(Gpio &gpio, int vdd, int vpp, int clock, int data_out, int data_in);
		virtual ~Breadboard();
		
		virtual void SetVddHigh(bool high);
		virtual void SetVppHigh(bool high);
		virtual void SetClockHigh(bool high);
		virtual void SetDataHigh(bool high);
		virtual bool GetDataHigh();
		
	private:
		Gpio &m_gpio;
		int m_vdd;
		int m_vpp;
		int m_clock;
		int m_data_out;
		int m_data_in;
};

#endif

