/*
	rpipic - A PIC Programmer for the Raspberry Pi
	Copyright (C) 2012  Nicholas Tomlinson, Gert van Loo, Dom Cobley
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/


#ifndef GPIO_HPP
#define GPIO_HPP

class Gpio
{
	public:
		virtual void Enable(int n, bool out) = 0;
		virtual void Disable(int n) = 0;
		virtual int Read(int n) = 0;
		virtual void Write(int n, bool high) = 0;
};

class FileGpio : public Gpio
{
	public:
		virtual void Enable(int n, bool out);
		virtual void Disable(int n);
		virtual int Read(int n);
		virtual void Write(int n, bool high);
		
	private:
		int ReadFile(const char *filename);
		void WriteFile(const char *filename, const char *value);
};




//  The DirectGpio class is based on stuff from:
//  http://elinux.org/RPi_Low-level_peripherals#GPIO_Driving_Example_.28C.29
//  Note: The original code has been modified somewhat
//
//  How to access GPIO registers from C-code on the Raspberry-Pi
//  Example program
//  15-January-2012
//  Dom and Gert
//

class DirectGpio : public Gpio
{
	public:
		DirectGpio();
		
		virtual void Enable(int n, bool out);
		virtual void Disable(int n);
		virtual int Read(int n);
		virtual void Write(int n, bool high);
		
	private:
		int  mem_fd;
		char *gpio_mem, *gpio_map;
		char *spi0_mem, *spi0_map;
		volatile unsigned *gpio;
};

#endif

